@@ -9,11 +9,11 @@ Array::Iterator - A simple class for iterating over Perl arrays
 
 =head1 VERSION
 
-Version 0.134
+Version 0.135
 
 =cut
 
-our $VERSION = '0.134';
+our $VERSION = '0.135';
 
 =head1 SYNOPSIS
 
@@ -66,7 +66,7 @@ no attempt is made to enforce this (although I will if I can find an efficient
 means of doing so). This class only intends to provide a clear and simple
 means of generic iteration, nothing more (yet).
 
-=head2 B<new (@array | $array_ref | $hash_ref)>
+=head2 new (@array | $array_ref | $hash_ref)
 
 The constructor can be passed either a plain Perl array, an array reference,
 or a hash reference (with the array specified as a single key of the hash,
@@ -86,15 +86,14 @@ of a HASH reference, with the key, __array__:
 sub new {
 	my ($_class, @array) = @_;
 
-	(@array)
-        || die 'Insufficient Arguments: you must provide something to iterate over';
+	(@array) || die 'Insufficient Arguments: you must provide something to iterate over';
+
 	my $class = ref($_class) || $_class;
 	my $_array;
 	if (scalar @array == 1) {
 		if (ref $array[0] eq 'ARRAY') {
 		    $_array = $array[0];
-		}
-		elsif (ref $array[0] eq 'HASH') {
+		} elsif (ref $array[0] eq 'HASH') {
 		    die 'Incorrect type: HASH reference must contain the key __array__'
 		        unless exists $array[0]->{__array__};
 		    die 'Incorrect type: __array__ value must be an ARRAY reference'
@@ -106,32 +105,34 @@ sub new {
 		$_array = \@array;
 	}
 	my $iterator = {
-        _current_index => 0,
-        _length => 0,
-        _iteratee => [],
-        _iterated => 0,
+		_current_index => 0,
+		_length => 0,
+		_iteratee => [],
+		_iterated => 0,	# -1 when going backwards, +1 when going forwards
         };
 	bless($iterator, $class);
 	$iterator->_init(scalar(@{$_array}), $_array);
 	return $iterator;
 }
 
-### methods
-
-# private methods
-
 sub _init {
 	my ($self, $length, $iteratee) = @_;
 	(defined($length) && defined($iteratee))
-        || die 'Insufficient Arguments: you must provide an length and an iteratee';
+		|| die 'Insufficient Arguments: you must provide an length and an iteratee';
 	$self->{_current_index} = 0;
 	$self->{_length} = $length;
-	$self->{_iteratee} = $iteratee;
+	# $self->{_iteratee} = $iteratee;
+
+	# Store a private copy to prevent modifications
+	$self->{_iteratee} = [@{$iteratee}];
 }
 
-# protected method
+=head2 _current_index
 
-# this can be used in a subclass to access the value
+An lvalue-ed subroutine that allows access to the iterator's internal pointer.
+This can be used in a subclass to access the value.
+
+=cut
 
 # We need to alter this so it's an lvalue
 sub _current_index : lvalue {
@@ -140,8 +141,13 @@ sub _current_index : lvalue {
     $_[0]->{_current_index}
 }
 
-# this we should never need to alter
-# so we don't make it a lvalue
+=head2 _iteratee
+
+This returns the item being iterated over, in our case an array.
+
+=cut
+
+# This we should never need to alter so we don't make it a lvalue
 sub _iteratee {
     (UNIVERSAL::isa((caller)[0], __PACKAGE__))
         || die 'Illegal Operation: This method can only be called by a subclass';
@@ -152,12 +158,20 @@ sub _iteratee {
 # to a protected one, and check our access
 # as well
 sub _getItem {
-    (UNIVERSAL::isa((caller)[0], __PACKAGE__))
-        || die 'Illegal Operation: This method can only be called by a subclass';
+	(UNIVERSAL::isa((caller)[0], __PACKAGE__)) || die 'Illegal Operation: This method can only be called by a subclass';
+
 	my ($self, $iteratee, $index) = @_;
 	return $iteratee->[$index];
 }
 
+=head2 _get_item ($iteratee, $index)
+
+This method is used by all other routines to access items. Given the iteratee
+and an index, it will return the item being stored in the C<$iteratee> at the index
+of C<$index>.
+
+=cut
+
 sub _get_item { my $self = shift; $self->_getItem(@_) }
 
 # we need to alter this so it's an lvalue
@@ -167,12 +181,18 @@ sub _iterated : lvalue {
     $_[0]->{_iterated}
 }
 
+=head2 iterated
+
+Access to the _iterated status, for subclasses
+
+=cut
+
 sub iterated {
     my ($self) = @_;
     return $self->{_iterated};
 }
 
-=head2 B<has_next([$n])>
+=head2 has_next([$n])
 
 This method returns a boolean. True (1) if there are still more elements in
 the iterator, false (0) if there are not.
@@ -193,18 +213,28 @@ Throws an exception if C<$n> E<lt>= 0.
 sub has_next {
 	my ($self, $n) = @_;
 
-    if(not defined $n) { $n = 1 }
-    elsif(not $n)      { die "has_next(0) doesn't make sense, did you mean current()?" }
-    elsif($n < 0)      { die "has_next() with negative argument doesn't make sense, perhaps you should use a BiDirectional iterator" }
+	if(not defined $n) {
+		$n = 1
+	} elsif(not $n) {
+		die "has_next(0) doesn't make sense, did you mean current()?"
+	} elsif($n < 0) {
+		die "has_next() with negative argument doesn't make sense, perhaps you should use a BiDirectional iterator"
+	}
 
-    my $idx = $self->{_current_index} + ($n - 1);
+	my $idx = $self->{_current_index} + ($n - 1);
 
 	return ($idx < $self->{_length}) ? 1 : 0;
 }
 
+=head2 hasNext
+
+Alternative name for has_next
+
+=cut
+
 sub hasNext { my $self = shift; $self->has_next(@_) }
 
-=head2 B<next>
+=head2 next
 
 This method returns the next item in the iterator, be sure to only call this
 once per iteration as it will advance the index pointer to the next item. If
@@ -214,14 +244,15 @@ will be thrown.
 =cut
 
 sub next {
-	my ($self) = @_;
-    ($self->{_current_index} < $self->{_length})
-        || die 'Out Of Bounds: no more elements';
+	my $self = shift;
+
+	($self->{_current_index} < $self->{_length}) || die 'Out Of Bounds: no more elements';
+
         $self->{_iterated} = 1;
 	return $self->_getItem($self->{_iteratee}, $self->{_current_index}++);
 }
 
-=head2 B<get_next>
+=head2 get_next
 
 This method returns the next item in the iterator, be sure to only call this
 once per iteration as it will advance the index pointer to the next item. If
@@ -251,9 +282,15 @@ sub get_next {
     return $self->_getItem($self->{_iteratee}, $self->{_current_index}++);
 }
 
+=head2 getNext
+
+Alternative name for get_next
+
+=cut
+
 sub getNext { my $self = shift; $self->get_next(@_) }
 
-=head2 B<peek([$n])>
+=head2 peek([$n])
 
 This method can be used to peek ahead at the next item in the iterator. It
 is non-destructive, meaning it does not advance the internal pointer. If
@@ -289,7 +326,7 @@ sub peek {
     return $self->_getItem($self->{_iteratee}, $idx);
 }
 
-=head2 B<current>
+=head2 current
 
 This method can be used to get the current item in the iterator. It is non-destructive,
 meaning that it does not advance the internal pointer. This value will match the
@@ -302,7 +339,7 @@ sub current {
 	return $self->_getItem($self->{_iteratee}, $self->currentIndex());
 }
 
-=head2 B<current_index>
+=head2 current_index
 
 This method can be used to get the current index in the iterator. It is non-destructive,
 meaning that it does not advance the internal pointer. This value will match the index
@@ -315,55 +352,49 @@ sub current_index {
 	return ($self->{_current_index} != 0) ? $self->{_current_index} - 1 : 0;
 }
 
+=head2 currentIndex
+
+Alternative name for current_index
+
+=cut
+
 sub currentIndex { my $self = shift; $self->current_index(@_) }
 
-sub get_length {
-    my ($self) = @_;
-    return $self->{_length};
+=head2 reset
+
+Reset index to allow iteration from the start
+
+=cut
+
+sub reset
+{
+	my $self = shift;
+	$self->{'_current_index'} = 0;
 }
 
+=head2 get_length
+
+This is a basic accessor for getting the length of the array being iterated over.
+
+=cut
+
+sub get_length {
+	my $self = shift;
+
+	return $self->{_length};
+}
+
+=head2 getLength
+
+Alternative name for get_length
+
+=cut
+
 sub getLength { my $self = shift; $self->get_length(@_) }
 
 1;
 
-=for Pod::Coverage .+
-
-=head1 METHODS
-
-=head2 Public Methods
-
-=item B<get_length>
-
-This is a basic accessor for getting the length of the array being iterated over.
-
-=back
-
-=head2 Protected Methods
-
-These methods are I<protected>, in the Java/C++ sense of the word. They can only be
-called internally by subclasses of Array::Iterator, an exception is thrown if that
-condition is violated. They are documented here only for people interested in
-subclassing Array::Iterator.
-
-=over 4
-
-=item B<_current_index>
-
-An lvalue-ed subroutine that allows access to the iterator's internal pointer.
-
-=item B<_iteratee>
-
-This returns the item being iterated over, in our case an array.
-
-=item B<_get_item ($iteratee, $index)>
-
-This method is used by all other routines to access items. Given the iteratee
-and an index, it will return the item being stored in the C<$iteratee> at the index
-of C<$index>.
-
-=back
-
-=head1 TO DO
+=head1 TODO
 
 =over 4
 
@@ -386,15 +417,15 @@ to Array::Iterator, they are:
 
 =over 4
 
-=item B<Array::Iterator::BiDirectional>
+=item C<Array::Iterator::BiDirectional>
 
 Adds the ability to move backward and forward through the array.
 
-=item B<Array::Iterator::Circular>
+=item C<Array::Iterator::Circular>
 
 When this iterator reaches the end of its list, it will loop back to the start again.
 
-=item B<Array::Iterator::Reusable>
+=item C<Array::Iterator::Reusable>
 
 This iterator can be reset to its beginning and used again.
 
@@ -415,7 +446,7 @@ you don't happen to like the way I do it.
 
 =over 4
 
-=item B<Tie::Array::Iterable>
+=item Tie::Array::Iterable
 
 This module ties the array, something we do not do. But it also makes an attempt to
 account for, and allow the array to be changed during iteration. It accomplishes this
@@ -427,7 +458,7 @@ In fact,
 it is recommended to not alter your array with Array::Iterator,
 and if possible we will enforce this in later versions.
 
-=item B<Data::Iter>
+=item Data::Iter
 
 This module allows for simple iteration over both hashes and arrays.
 It does it by
@@ -435,7 +466,7 @@ importing several functions that can be used to loop over either type (hash or a
 in the same way. It is an interesting module, it differs from Array::Iterator in
 paradigm (Array::Iterator is more OO) and intent.
 
-=item B<Class::Iterator>
+=item Class::Iterator
 
 This is essentially a wrapper around a closure-based iterator.
 This method can be very
@@ -444,7 +475,7 @@ closures. I actually was a closure-as-iterator fan for a while but eventually mo
 away from it in favor of the more plain vanilla means of iteration, like that found
 Array::Iterator.
 
-=item B<Class::Iter>
+=item Class::Iter
 
 This is part of the Class::Visitor module and is a Visitor and Iterator extension to
 Class::Template.
@@ -485,6 +516,10 @@ L<http://www.iinteractive.com>
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+=head1 PREVIOUS MAINTAINER
+
+Maintained 2017 to 2025 PERLANCAR
+
 =head1 SUPPORT
 
 This module is provided as-is without any warranty.
