@@ -1,10 +1,10 @@
-
 package Array::Iterator::BiDirectional;
 
 use strict;
 use warnings;
 
 use Array::Iterator;
+use Carp;
 
 # AUTHORITY
 # DATE
@@ -12,68 +12,16 @@ use Array::Iterator;
 
 =head1 VERSION
 
-Version 0.134
+Version 0.135
 
 =cut
 
-our $VERSION = '0.134';
-
-our @ISA = qw(Array::Iterator);
-
-sub has_previous {
-	my ($self, $n) = @_;
-
-    if(not defined $n) { $n = 1 }
-    elsif(not $n)      { die "has_previous(0) doesn't make sense, did you mean current()?" }
-    elsif($n < 0)      { die "has_previous() with negative argument doesn't make sense, did you mean has_next()?" }
-
-    my $idx = $self->_current_index - $n;
-
-	return ($idx > 0) ? 1 : 0;
-}
-
-sub hasPrevious { my $self = shift; $self->has_previous(@_) }
-
-sub previous {
-	my ($self) = @_;
-    (($self->_current_index - 1) > 0)
-        || die "Out Of Bounds : no more elements";
-        $self->_iterated = 1;
-	return $self->_getItem($self->_iteratee, --$self->_current_index);
-}
-
-sub get_previous {
-    my ($self) = @_;
-    return undef unless (($self->_current_index - 1) > 0); ## no critic: Subroutines::ProhibitExplicitReturnUndef
-    $self->_iterated = 1;
-    return $self->_getItem($self->_iteratee, --$self->_current_index);
-}
-
-sub getPrevious { my $self = shift; $self->get_previous(@_) }
-
-sub look_back {
-    my ($self, $n) = @_;
-
-    if(not defined $n) { $n = 1 }
-    elsif(not $n)      { die "look_back(0) doesn't make sense, did you mean get_previous()?" }
-    elsif($n < 0)      { die "look_back() with negative argument doesn't make sense, did you mean get_next()?" }
-
-    my $idx = $self->_current_index - ($n + 1);
-
-    return undef unless ($idx > 0); ## no critic: Subroutines::ProhibitExplicitReturnUndef
-    $self->_iterated = 1;
-    return $self->_getItem($self->_iteratee, $idx);
-}
-
-sub lookBack { my $self = shift; $self->look_back(@_) }
-
-1;
-#ABSTRACT: A subclass of Array::Iterator to allow forwards and backwards iteration
-
-=for Pod::Coverage .+
+our $VERSION = '0.135';
 
 =head1 SYNOPSIS
 
+Occasionally it is useful for an iterator to go in both directions, forward and backward. One example would be token processing. When looping though tokens it is sometimes necessary to advance forward looking for a match to a rule. If the match fails, a bi-directional iterator can be moved back so that the next rule can be tried.
+
   use Array::Iterator::BiDirectional;
 
   # create an instance of the iterator
@@ -90,9 +38,73 @@ sub lookBack { my $self = shift; $self->look_back(@_) }
       }
   }
 
-=head1 DESCRIPTION
+=cut
 
-Occasionally it is useful for an iterator to go in both directions, forward and backward. One example would be token processing. When looping though tokens it is sometimes necessary to advance forward looking for a match to a rule. If the match fails, a bi-directional iterator can be moved back so that the next rule can be tried.
+our @ISA = qw(Array::Iterator);
+
+sub has_previous {
+	my ($self, $n) = @_;
+
+	if(not defined $n) { $n = 1 }
+	elsif(not $n)      { die "has_previous(0) doesn't make sense, did you mean current()?" }
+	elsif($n < 0)      { die "has_previous() with negative argument doesn't make sense, did you mean has_next()?" }
+
+	my $idx = $self->_current_index - $n;
+
+	if(!defined($self->{_iterated}) || ($self->{_iterated} >= 0)) {
+		return ($idx > 0) ? 1 : 0;
+	}
+	return ($idx >= 0) ? 1 : 0;
+}
+
+sub hasPrevious { my $self = shift; $self->has_previous(@_) }
+
+sub previous {
+	my $self = shift;
+
+	if($self->{'_iterated'} >= 0) {
+		(($self->_current_index - 1) > 0) || Carp::croak('previous: Out Of Bounds: no more elements');
+	} else {
+		(($self->_current_index - 1) >= 0) || Carp::croak('previous: Out Of Bounds: no more elements');
+	}
+	$self->_iterated = -1;
+	return $self->_getItem($self->_iteratee, --$self->_current_index);
+}
+
+sub get_previous
+{
+	my $self = shift;
+
+	return undef unless $self->hasPrevious(); ## no critic: Subroutines::ProhibitExplicitReturnUndef
+	if($self->_iterated == 1) {	# RT126034
+		--$self->{_current_index};
+	}
+	$self->_iterated = -1;
+	return undef unless $self->hasPrevious(); ## no critic: Subroutines::ProhibitExplicitReturnUndef
+	return $self->_getItem($self->_iteratee, --$self->{_current_index});
+}
+
+sub getPrevious { my $self = shift; $self->get_previous(@_) }
+
+sub look_back {
+    my ($self, $n) = @_;
+
+    if(not defined $n) { $n = 1 }
+    elsif(not $n)      { die "look_back(0) doesn't make sense, did you mean get_previous()?" }
+    elsif($n < 0)      { die "look_back() with negative argument doesn't make sense, did you mean get_next()?" }
+
+    my $idx = $self->_current_index - ($n + 1);
+
+    return undef unless ($idx > 0); ## no critic: Subroutines::ProhibitExplicitReturnUndef
+    return $self->_getItem($self->_iteratee, $idx);
+}
+
+sub lookBack { my $self = shift; $self->look_back(@_) }
+
+1;
+#ABSTRACT: A subclass of Array::Iterator to allow forwards and backwards iteration
+
+=for Pod::Coverage .+
 
 =head1 METHODS
 
